@@ -1,7 +1,7 @@
 try:
     import torch
-except ImportError:
-    raise ImportError('torch_tools is needed pytorch!')
+except ModuleNotFoundError:
+    raise ModuleNotFoundError('torch_tools is needed pytorch!')
 
 from processingtools.torch_tools.torch_function import *
 from processingtools.torch_tools.TorchProcess import *
